"""
   Copyright 2021 DataDistillr Inc.
   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at
       http://www.apache.org/licenses/LICENSE-2.0
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""


class AuthorizationException(Exception):
    """Exception raised for unauthorized attempts to access DataDistillr

    Attributes:
        url - Url which threw the error
        message - Explanation of the error
    """

    def __init__(self, url, message):
        self.url = url
        self.message = message
        super().__init__(self.message)

    def __str__(self):
        msg = self.message
        msg_type = "Authentication Error: You are not authorized to access this resource: "
        url = self.url
        return repr(f"{msg} {msg_type} {url}")



"""
This file is for testing the datadistillr API calls.
"""
import unittest
import requests
import datadistillr as ddr
requests.packages.urllib3.disable_warnings()


class DatadistillrApiTest(unittest.TestCase):
    """
    This class is for testing the datadistillr API calls.
    """

    def test_failed_api_call(self):
        """
        Tests that API call returns authorization error message if authorization token is incorrect.
        """

        url = "https://app.datadistillr.io/v1/results/298294315"
        auth = "no_auth"
        try:
            ddr.Datadistillr.get_dataframe(url, auth)
        except ddr.AuthorizationException as exception:
            self.assertEqual(exception.url, url)
            self.assertIn("You are not authorized to access this resource", str(exception))

    def test_successful_call(self):
        """
        Tests that API call returns expected values.
        """

        url = "https://app.datadistillr.io/v1/results/298294315"
        auth_token = "DDR1-HMAC-SHA256 Credential=cb161e29b3baf02c26a25b2809ebb1e3 " \
                     "Signature=19488068ba57273290a924896662fdfa50f89b7e0b8558e165d94e4582f3c082"
        data_frame = ddr.Datadistillr.get_dataframe(url, auth_token)
        self.assertEqual(data_frame['col_1'].count(), 11)
        self.assertEqual(data_frame['January'].count(), 11)
        self.assertEqual(data_frame.shape, (11, 2))


if __name__ == '__main__':
    unittest.main()



"""
This file defines the class for getting account level data from Datadistillr account.
"""

import requests
from datadistillr.project import Project


class DatadistillrAccount:
    """
    This is a class for getting account level data from Datadistillr account.

    Attributes:
        email (string): The email linked to Datadistillr account.
        password (string): The password linked to Datadistillr account.
    """

    BASE_URL = "https://app.datadistillr.io/api/"
    LOGIN_PAGE = BASE_URL + "login"
    ORGANIZATIONS_LIST = BASE_URL + "organization"
    LOGOUT_PAGE = BASE_URL + 'logout'
    PROJECT_DISTILLRY = BASE_URL + "projectDistillry"

    def __init__(self, email, password):
        """
        The constructor for the DatadistillrAccount class. Creates a session.

        Parameters:
            email (string): The email linked to Datadistillr account.
            password (string): The password linked to Datadistillr account.
        """
        requests.packages.urllib3.disable_warnings()
        # stores cookies, so you can make requests without multiple logins (pass around cookie)
        self.session = requests.Session()
        self.email = email
        self.password = password
        self.login_resp_json = self._login()
        self.is_logged_in = self.login_resp_json["loggedIn"]
        self.proj_token_dict = {}

    def _login(self):
        """
        Login and authenticate to DataDistillr.

        Returns:
            json: A json containing account details and login status.
        """

        user_info = {
            "email": self.email,
            "password": self.password,
            "invitations": {
                "organizationInvitationToken": None,
                "projectInvitationToken": None,
                "teamInvitationToken": None}
        }
        login_response = self.session.post(url=self.LOGIN_PAGE, json=user_info, verify=False)
        login_resp_json = login_response.json()
        return login_resp_json

    def logout(self):
        """
        Log user out of DataDistillr account.

        Returns:
            json: A json containing account details and login status.
        """

        logout_response = self.session.get(url=self.LOGOUT_PAGE, verify=False)
        logout_resp_json = logout_response.json()
        self.is_logged_in = logout_resp_json["loggedIn"]
        return logout_resp_json

    def get_project_token_dict(self):
        """
        Returns dictionary with project tokens as keys and project names as values.

        Returns:
            dictionary (int -> str): dictionary with project token as key and project name as value.
        """

        # check if login is correct
        if not self.is_logged_in:
            raise Exception("login is incorrect")

        org_token = self.login_resp_json["activeOrganization"]["token"]
        projects_page = self.BASE_URL + "organization/" + str(org_token) + "/projects"
        projects_response = self.session.get(url=projects_page, verify=False)

        # Converts response to JSON
        proj_resp_json = projects_response.json()

        # Gets the projects list
        proj_list = proj_resp_json["projects"]

        for proj in proj_list:
            # Creates a dictionary of all projects and their tokens
            self.proj_token_dict[proj["token"]] = proj["name"]
        return self.proj_token_dict

    def get_project_token(self, project_name):
        """
        Returns project token that matches project_name

        Returns:
            int: project token
        """

        proj_token_dict = self.get_project_token_dict()
        for token, name in proj_token_dict.items():
            if project_name == name:
                return token
        raise Exception("token not found")

    def get_project(self, project_token):
        """
        Returns individual project object.

        Parameters:
            project_token (int): Token that uniquely identifies project. A dictionary with all
            project tokens can be found using the get_project_token_dict() function.

        Returns:
            project: A project object.
        """

        # check if login is correct
        if not self.is_logged_in:
            return "login is incorrect"

        project_details_page = self.PROJECT_DISTILLRY + "/" + str(project_token)
        # Gets the url
        project_details = self.session.get(url=project_details_page)
        # Parses the response from JSON to a python dictionary
        project_details_json = project_details.json()['project']
        proj_object = Project(project_details_json, self.session)
        # Returns the parsed JSON
        return proj_object

    def get_projects(self):
        """
        Returns all projects in DataDistillr account.

        Returns:
            list<Project>: A list of project objects.
        """

        if not self.is_logged_in:
            raise Exception("login is incorrect")

        proj_object_list = []

        project_tokens = self.get_project_token_dict().keys()
        for project_token in project_tokens:
            proj_object = self.get_project(project_token)
            proj_object_list.append(proj_object)

        return proj_object_list

    def get_organizations(self):
        """
        Returns all organizations that the user has access to.

        Returns:
            list: A list of organizations that the user has access to.
        """

        if not self.is_logged_in:
            raise Exception("login is incorrect")

        organizations_response = self.session.get(url=self.ORGANIZATIONS_LIST, verify=False)
        organizations_resp_json = organizations_response.json()
        return organizations_resp_json["organizations"]



"""
This file defines the class for testing the DatadistillrAccount class.
"""

import unittest
import responses
from datadistillr.datadistillr_account import DatadistillrAccount
from datadistillr.project import Project


class TestDatadistillrAccount(unittest.TestCase):
    """
    This class is for testing the DatadistillrAccount class.
    """

    BASE_URL = "https://app.datadistillr.io/api/"
    ORGANIZATIONS_ROUTE = BASE_URL + "organization"
    PROJECT_DISTILLRY_ROUTE = BASE_URL + "projectDistillry"
    # Real datadistillr account information, cannot be mocked because of authentication limitations
    MOCK_ORG_TOKEN = 880610291

    # Fake datadistillr account information
    MOCK_PROJ1_NAME = "Project 1"
    MOCK_PROJ1_TOKEN = 111111111
    MOCK_PROJ2_NAME = "Project 2"
    MOCK_PROJ2_TOKEN = 222222222

    MOCK_PROJ1_ROUTE_RESP = {'project': {'name': MOCK_PROJ1_NAME, 'token': MOCK_PROJ1_TOKEN}}
    MOCK_PROJ2_ROUTE_RESP = {'project': {'name': MOCK_PROJ2_NAME, 'token': MOCK_PROJ2_TOKEN}}
    MOCK_PROJS_ROUTE_RESP = {'projects': [MOCK_PROJ1_ROUTE_RESP['project'],
                                          MOCK_PROJ2_ROUTE_RESP['project']]}
    MOCK_ORGS_ROUTE_RESP = {'organizations': [{'name': 'hidden', 'token': 880610291}]}

    def __init__(self, method_name: str) -> None:
        """
         The constructor for the TestDatadistillrAccount class.

         Parameters:
             method_name (string): Test function to be run.
         """
        super().__init__(methodName=method_name)

        # init DatadistillrAccount object
        email = 'Amandaha29@gmail.com'
        password = 'Password1!'
        self.datadistillr_account = DatadistillrAccount(email, password)

    def test_login(self):
        """
        Tests that DataDistillrAccount is authenticated once initialized.
        """

        self.assertTrue(self.datadistillr_account.is_logged_in)

    def test_logout(self):
        """
        Tests that DataDistillrAccount cannot access functions if logged out.
        """

        self.datadistillr_account.logout()
        self.assertFalse(self.datadistillr_account.is_logged_in)
        self.assertRaises(Exception, self.datadistillr_account.get_projects)
        self.assertRaises(Exception, self.datadistillr_account.get_project_token_dict)
        self.assertRaises(Exception, self.datadistillr_account.get_project)
        self.assertRaises(Exception, self.datadistillr_account.get_organizations)

    @responses.activate
    def test_get_projects(self):
        """
        Tests that get_projects() returns list of Project objects with expected names.
        """

        projs_route = self.BASE_URL + "organization/" + str(self.MOCK_ORG_TOKEN) + "/projects"
        proj1_route = self.PROJECT_DISTILLRY_ROUTE + "/" + str(self.MOCK_PROJ1_TOKEN)
        proj2_route = self.PROJECT_DISTILLRY_ROUTE + "/" + str(self.MOCK_PROJ2_TOKEN)

        # register mock response
        responses.add(responses.GET, projs_route, json=self.MOCK_PROJS_ROUTE_RESP, status=200)
        responses.add(responses.GET, proj1_route, json=self.MOCK_PROJ1_ROUTE_RESP, status=200)
        responses.add(responses.GET, proj2_route, json=self.MOCK_PROJ2_ROUTE_RESP, status=200)

        # test get_projects() function
        projects = self.datadistillr_account.get_projects()
        assert all(isinstance(proj, Project) for proj in projects)
        self.assertEqual(projects[0].name, self.MOCK_PROJ1_NAME)
        self.assertEqual(projects[1].name, self.MOCK_PROJ2_NAME)

    @responses.activate
    def test_get_project_token_dict(self):
        """
        Tests that get_project_token_dict() returns dictionary with expected tokens and names
        """

        projs_route = self.BASE_URL + "organization/" + str(self.MOCK_ORG_TOKEN) + "/projects"

        # register mock response
        responses.add(responses.GET, projs_route, json=self.MOCK_PROJS_ROUTE_RESP, status=200)

        # test get_project_token_dict() function
        project_token_dict = self.datadistillr_account.get_project_token_dict()
        project_tokens = list(project_token_dict.keys())
        project_names = list(project_token_dict.values())
        self.assertEqual(project_tokens, [self.MOCK_PROJ1_TOKEN, self.MOCK_PROJ2_TOKEN])
        self.assertEqual(project_names, [self.MOCK_PROJ1_NAME, self.MOCK_PROJ2_NAME])

    @responses.activate
    def test_get_project_token(self):
        """
        Tests that get_project_token() returns token matching project name
        """

        projs_route = self.BASE_URL + "organization/" + str(self.MOCK_ORG_TOKEN) + "/projects"

        # register mock response
        responses.add(responses.GET, projs_route, json=self.MOCK_PROJS_ROUTE_RESP, status=200)

        # test get_project_token() function
        project_token = self.datadistillr_account.get_project_token(self.MOCK_PROJ1_NAME)
        self.assertEqual(project_token, self.MOCK_PROJ1_TOKEN)

    @responses.activate
    def test_get_project(self):
        """
        Tests that get_project() returns a project object with expected name.
        """

        proj1_route = self.PROJECT_DISTILLRY_ROUTE + "/" + str(self.MOCK_PROJ1_TOKEN)

        # register mock response
        responses.add(responses.GET, proj1_route, json=self.MOCK_PROJ1_ROUTE_RESP, status=200)

        # test get_project() function
        project = self.datadistillr_account.get_project(self.MOCK_PROJ1_TOKEN)
        self.assertIsInstance(project, Project)
        self.assertEqual(project.name, self.MOCK_PROJ1_NAME)

    @responses.activate
    def test_get_organizations(self):
        """
        Tests that get_organizations() returns a list of organization dictionaries.
        """

        # register mock response
        responses.add(responses.GET, self.ORGANIZATIONS_ROUTE, json=self.MOCK_ORGS_ROUTE_RESP,
                      status=200)

        # test get_organizations() function
        organizations = self.datadistillr_account.get_organizations()
        self.assertEqual(organizations, self.MOCK_ORGS_ROUTE_RESP['organizations'])



"""
This file defines the class for testing the Project class.
"""
import json
import unittest
import responses
from responses import matchers
from datadistillr.datadistillr_account import DatadistillrAccount


class TestProject(unittest.TestCase):
    """
    This class is for testing the Project class.
    """
    BASE_URL = "https://app.datadistillr.io/api/"
    QUERY_BARRELS_ROUTE = BASE_URL + "queryBarrels"
    QUERY_RESULTS_ROUTE = BASE_URL + "queryResults"
    PROJECT_ROUTE = BASE_URL + "project"
    DATA_SOURCE_ROUTE = BASE_URL + "dataSource"

    # Real datadistillr account information, cannot be mocked because of authentication limitations
    MOCK_PROJECT_NAME = "Test Project 2"
    MOCK_PROJECT_TOKEN = 347151952
    MOCK_TAB_NAME = "months"
    MOCK_TAB_TOKEN = 298294315

    # Fake datadistillr account information
    MOCK_BARREL_TOKEN = 111111111
    MOCK_QUERY_TOKEN = 222222222
    MOCK_RUN_REQUEST_TOKEN = 333333333
    MOCK_DATASOURCE_TOKEN = 444444444
    MOCK_DATASOURCE_NAME = "data source"

    QUERY_RUN_ROUTE = QUERY_BARRELS_ROUTE + "/" + str(MOCK_BARREL_TOKEN) + "/query/" + \
        str(MOCK_QUERY_TOKEN) + "/run"

    QUERY_RESULTS_ROUTE = QUERY_RESULTS_ROUTE + "/" + str(MOCK_RUN_REQUEST_TOKEN)

    MOCK_QUERY_RUN_ROUTE_RESP = {
        'query': {'query': '--',
                  'token': MOCK_QUERY_TOKEN,
                  'queryBarrelToken': MOCK_BARREL_TOKEN},
        'requestToken': MOCK_RUN_REQUEST_TOKEN
    }

    MOCK_QUERY_RESULTS_ROUTE_RESP = {
        'results': [['1', 'January'], ['2', 'February'], ['3', 'March']],
        'summary': {'columnNames': ['Index', 'Month'], 'dataTypes': ['VARCHAR', 'VARCHAR'],
                    'rowsPerPage': 500, 'totalNumRows': 11, 'page': 1,
                    'totalPages': 1}, 'queryRun': {'app_id': 0, 'status': 'complete',
                                                   'token': MOCK_RUN_REQUEST_TOKEN,
                                                   'queryToken': MOCK_QUERY_TOKEN,
                                                   'numRows': 11}}

    def __init__(self, method_name: str) -> None:
        """
         The constructor for the TestProject class.

         Parameters:
             method_name (string): Test function to be run.
         """
        super().__init__(methodName=method_name)

        email = 'Amandaha29@gmail.com'
        password = 'Password1!'
        self.datadistillr_account = DatadistillrAccount(email, password)

        project_token = list(self.datadistillr_account.get_project_token_dict().keys())[0]
        self.project = self.datadistillr_account.get_project(project_token)

    @responses.activate
    def test_get_tab_token_dict(self):
        """
        Tests that get_tab_token_dict() returns dictionary of integers and strings.
        """

        tab_token_dict = self.project.get_tab_token_dict()
        tokens = tab_token_dict.keys()
        names = tab_token_dict.values()

        assert all(isinstance(token, int) for token in tokens)
        assert all(isinstance(name, str) for name in names)

    @responses.activate
    def test_get_tab_token(self):
        """
        Tests that get_tab_token() returns token matching tab name
        """

        # test get_tab_token() function
        tab_token = self.project.get_tab_token(self.MOCK_TAB_NAME)
        self.assertEqual(tab_token, self.MOCK_TAB_TOKEN)

    @responses.activate
    def test_execute_existing_query(self):
        """
        Tests that execute_existing_query() returns a dataframe.
        """

        mock_query_barrel_route_resp = {
            'queryBarrel': {
                'queries': [{'token': self.MOCK_QUERY_TOKEN,
                             'queryBarrelToken': self.MOCK_BARREL_TOKEN }],
                'token': self.MOCK_BARREL_TOKEN
            }
        }

        query_barrel_route = self.QUERY_BARRELS_ROUTE + "/" + str(self.MOCK_BARREL_TOKEN)

        # registering mock responses
        responses.add(
            method=responses.GET,
            url=query_barrel_route,
            json=mock_query_barrel_route_resp,
            status=200
        )
        responses.add(
            method=responses.GET,
            url=self.QUERY_RUN_ROUTE,
            json=self.MOCK_QUERY_RUN_ROUTE_RESP,
            status=200
        )
        responses.add(
            method=responses.GET,
            url=self.QUERY_RESULTS_ROUTE,
            json=self.MOCK_QUERY_RESULTS_ROUTE_RESP,
            status=200
        )

        # testing execute_existing_query function
        query_results_df = self.project.execute_existing_query(self.MOCK_BARREL_TOKEN)
        self.assertEqual(type(query_results_df).__name__, 'DataFrame')
        self.assertEqual(query_results_df['Index'].count(), 3)
        self.assertEqual(query_results_df['Month'].count(), 3)
        self.assertEqual(query_results_df.shape, (3, 2))

    @responses.activate
    def test_execute_new_query(self):
        """
        Tests that execute_new_query() returns a dataframe.
        """
        mock_query_barrel_name = "query barrel name"
        mock_query = "SQL QUERY"

        mock_new_query_barrel_route_resp = {
            'queryBarrel': {
                'name': mock_query_barrel_name,
                'queries': [
                    {'query': mock_query,
                     'token': self.MOCK_QUERY_TOKEN,
                     'queryBarrelToken': self.MOCK_BARREL_TOKEN}
                ],
                'query': mock_query,
                'token': self.MOCK_BARREL_TOKEN
            }
        }

        mock_new_query_barrel_details = {
            "projectSlug": self.MOCK_PROJECT_NAME.lower().replace(' ', '-'),
            "projectToken": self.MOCK_PROJECT_TOKEN,
            "name": mock_query_barrel_name,
            "active": True,
            "icon": "type-icon-file",
            "query": "  " + mock_query
        }

        # registering mock responses
        responses.add(
            method=responses.POST,
            url=self.QUERY_BARRELS_ROUTE,
            body=json.dumps(mock_new_query_barrel_route_resp, indent=2).encode('utf-8'),
            match=[matchers.json_params_matcher(mock_new_query_barrel_details)],
        )
        responses.add(
            method=responses.GET,
            url=self.QUERY_RUN_ROUTE,
            json=self.MOCK_QUERY_RUN_ROUTE_RESP,
            status=200
        )
        responses.add(
            method=responses.GET,
            url=self.QUERY_RESULTS_ROUTE,
            json=self.MOCK_QUERY_RESULTS_ROUTE_RESP,
            status=200
        )

        # testing execute_existing_query function
        query_results_df = self.project.execute_new_query(mock_query_barrel_name, mock_query)
        self.assertEqual(type(query_results_df).__name__, 'DataFrame')
        self.assertEqual(query_results_df['Index'].count(), 3)
        self.assertEqual(query_results_df['Month'].count(), 3)
        self.assertEqual(query_results_df.shape, (3, 2))

    @responses.activate
    def test_get_data_source_token_dict(self):
        """
        Tests that get_data_source_token_dict() returns dictionary with expected tokens and names
        """

        mock_data_sources_route_resp = {'dataSources': [{'name': self.MOCK_DATASOURCE_NAME,
                                                         'token': self.MOCK_DATASOURCE_TOKEN}]}

        data_source_route = self.PROJECT_ROUTE + "/" + str(self.MOCK_PROJECT_TOKEN) + "/dataSource"

        # register mock response
        responses.add(
            method=responses.GET,
            url=data_source_route,
            json=mock_data_sources_route_resp,
            status=200
        )

        # test get_data_source_token_dict() function
        data_source_token_dict = self.project.get_data_source_token_dict()
        data_source_tokens = list(data_source_token_dict.keys())
        data_source_names = list(data_source_token_dict.values())
        self.assertEqual(data_source_tokens, [self.MOCK_DATASOURCE_TOKEN])
        self.assertEqual(data_source_names, [self.MOCK_DATASOURCE_NAME])

    @responses.activate
    def test_get_data_source_token(self):
        """
        Tests that get_data_source_token() returns token matching data source name
        """

        mock_data_sources_route_resp = {'dataSources': [{'name': self.MOCK_DATASOURCE_NAME,
                                                         'token': self.MOCK_DATASOURCE_TOKEN}]}

        data_source_route = self.PROJECT_ROUTE + "/" + str(self.MOCK_PROJECT_TOKEN) + "/dataSource"

        # register mock response
        responses.add(
            method=responses.GET,
            url=data_source_route,
            json=mock_data_sources_route_resp,
            status=200
        )

        # test get_data_source_token() function
        data_source_token = self.project.get_data_source_token(self.MOCK_DATASOURCE_NAME)
        self.assertEqual(data_source_token, self.MOCK_DATASOURCE_TOKEN)

    @responses.activate
    def test_upload_files(self):
        """
        Tests that upload_files() uploads files successfully
        """

        mock_file_paths = ["datadistillr/tests/weekdays.csv", "datadistillr/tests/months.csv"]
        mock_files_to_upload = {'files': [{'name': 'weekdays.csv', 'size': 81, 'type': 'text/csv',
                                           'path': '/'},
                                          {'name': 'months.csv', 'size': 113, 'type': 'text/csv',
                                           'path': '/'}]
                                }

        mock_presigned_url1 = 'https://s3.amazonaws.com/prod.uploads.datadistillr.io/uploads' \
                              '/65b6fa8d/weekdays.csv?X-Amz-Content-Sha256=UNSIGNED-PAYLOAD&X-Amz' \
                              '-Algorithm=AWS4-HMAC-SHA256&X-Amz-Credential=AKIA2BUAV67D6YOG3MHJ' \
                              '%2F20220502%2Fus-east-1%2Fs3%2Faws4_request&X-Amz-Date' \
                              '=20220502T050421Z&X-Amz-SignedHeaders=host&X-Amz-Expires=30&X-Amz' \
                              '-Signature' \
                              '=b1de86d8c53e5ed23f4c204a15ce59992e19cffb7428c3c0503cb32bcc97b3d2 '

        mock_presigned_url2 = 'https://s3.amazonaws.com/prod.uploads.datadistillr.io/uploads' \
                              '/65b6fa8d/months.csv?X-Amz-Content-Sha256=UNSIGNED-PAYLOAD&X-Amz' \
                              '-Algorithm=AWS4-HMAC-SHA256&X-Amz-Credential=AKIA2BUAV67D6YOG3MHJ' \
                              '%2F20220502%2Fus-east-1%2Fs3%2Faws4_request&X-Amz-Date' \
                              '=20220502T050422Z&X-Amz-SignedHeaders=host&X-Amz-Expires=30&X-Amz' \
                              '-Signature' \
                              '=80b889d475b16d7b9bd7519dc7bbb61cfd09d2696b59742d487e4077cc7d76f0 '

        mock_upload_file_resp = {'presignedUrls': [mock_presigned_url1, mock_presigned_url2]}

        upload_file_route = self.DATA_SOURCE_ROUTE + "/" + str(self.MOCK_DATASOURCE_TOKEN) + "/file"

        # registering mock responses
        responses.add(
            method=responses.POST,
            url=upload_file_route,
            body=json.dumps(mock_upload_file_resp, indent=2).encode('utf-8'),
            match=[matchers.json_params_matcher(mock_files_to_upload)]
        )
        responses.add(
            method=responses.PUT,
            url=mock_presigned_url1,
            body="".encode('utf-8'),
            match=[matchers.header_matcher({'content-type': 'text/plain'})]
        )
        responses.add(
            method=responses.PUT,
            url=mock_presigned_url2,
            body="".encode('utf-8'),
            match=[matchers.header_matcher({'content-type': 'text/plain'})]
        )

        # test test_upload_files() function
        upload_file_resp = self.project.upload_files(self.MOCK_DATASOURCE_TOKEN, mock_file_paths)
        self.assertEqual(upload_file_resp, 'file uploaded successfully')



#!/usr/bin/env python
"""
   Copyright 2021 DataDistillr Inc.
   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at
       https://www.apache.org/licenses/LICENSE-2.0
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""
from pathlib import Path
from setuptools import setup, find_packages
this_directory = Path(__file__).parent

setup_args = dict(
    name='datadistillr',
    version='1.0.1',
    author='Charles Givre, Amanda Ha, Andrea Ha, Sanaa Mironov',
    author_email='charles@datadistillr.com',
    packages=find_packages(include=['datadistillr', 'datadistillr.*']),
    url='https://github.com/datadistillr/datadistillr-python-sdk',
    license="Apache",
    description='A Python SDK for interacting with datasets created on DataDistillr',
    long_description = (this_directory / "README.md").read_text(encoding='utf-8'),
    long_description_content_type="text/markdown",
    install_requires=[
        "pandas",
        "requests",
        "urllib3"
    ],
    classifiers=[
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: SQL',
        'Operating System :: OS Independent',
        'Topic :: Database'
    ]
)


def main():
    """
    Runs the setup of DataDistillr.
    :return: Nothing
    """
    setup(**setup_args)


if __name__ == '__main__':
    main()



"""
This file defines the project class for getting project level data.
"""
import time
import os
import ntpath
import pandas as pd


class Project:
    """
    This is a class for getting project level data.

    Attributes:
        proj_details (json): json containing details of project.
    """
    BASE_URL = "https://app.datadistillr.io/api/"
    PROJECT_DISTILLRY = BASE_URL + "projectDistillry"
    QUERY_BARRELS = BASE_URL + "queryBarrels"
    QUERY_RUN_PAGE = BASE_URL + "queryResults"
    PROJECT_PAGE = BASE_URL + "project"
    DATA_SOURCE_PAGE = BASE_URL + "dataSource"
    MAX_ATTEMPTS = 20
    SLEEP_TIMER = 5.0

    def __init__(self, proj_details, _curr_session):
        """
        The constructor for Datadistillr class. Creates a session and contains project details.

        Parameters:
            proj_details (JSON): JSON containing details of project.
        """

        self.session = _curr_session
        self.details_json = proj_details
        self.name = self.details_json["name"]
        self.project_token = self.details_json["token"]
        self.barrel_token_dict = {}
        self.data_source_token_dict = {}

    def get_tab_token_dict(self):
        """
        Returns dictionary with tab tokens as keys and tab names as values.
        A tab in the DataDistillr user interface is equivalent to a query barrel in API routes and
        responses.

        Returns:
            dictionary (int -> str): dictionary with query barrel token as key and query barrel
            name as value.
        """

        project_query_barrels = self.details_json["queryBarrels"]
        for query_barrel in project_query_barrels:
            self.barrel_token_dict[query_barrel["token"]] = query_barrel["name"]
        return self.barrel_token_dict

    def get_tab_token(self, tab_name):
        """
        Returns tab token that matches tab_name

        Returns:
            int: tab token
        """

        tab_token_dict = self.get_tab_token_dict()
        for token, name in tab_token_dict.items():
            if tab_name == name:
                return token
        raise Exception("token not found")

    def _get_recent_query_token(self, barrel_token):
        """
        Returns token of most recent query in query barrel.
        A tab in the DataDistillr user interface is equivalent to a query barrel in API routes and
        responses.

        Parameters:
            barrel_token (int): Token that uniquely identifies query barrel. A dictionary with all
            query barrel tokens can be found using the get_tab_token_dict() function.

        Returns:
            int: Token of most recent query in query barrel.
        """

        queries_page = self.QUERY_BARRELS + "/" + str(barrel_token)
        queries_response = self.session.get(url=queries_page)
        queries_response_json = queries_response.json()
        # Finds the part regarding the queries
        queries_list = queries_response_json["queryBarrel"]["queries"]
        # Finds token of most recent query
        query_token = queries_list[-1]["token"]
        return query_token

    def _get_query_results(self, url_endpoint: str, attempts: int) -> dict:
        """
        Returns results of previously ran query.

        Parameters:
            url_endpoint (str): API endpoint for query data

        Returns:
            pandas dataframe: Results of query.
        """

        results: dict = {'data': [], 'summary': {}}

        response = self.session.get(url=url_endpoint)

        # Grab JSON object from response
        response_data = response.json()

        # response is success and has data
        if response_data['queryRun']['status'] == 'complete':
            # add data from response to data list
            results['data'].extend(response_data['results'])
            results['summary'] = response_data['summary']

            # if response has a nextPage set... grab next page
            if response_data['summary'].get('nextPage', None) is not None:
                page_data = self._get_query_results(response_data['summary']['nextPage'],
                                                    attempts)
                results['data'].extend(page_data['data'])

                # deletes page, nextPage, and totalPages within summary
                try:
                    del results['summary']['page']
                except KeyError:
                    pass

                try:
                    del results['summary']['nextPage']
                except KeyError:
                    pass
                try:
                    del results['summary']['totalPages']
                except KeyError:
                    pass

        # Data request is still processing/running. Will try in a few seconds
        elif response_data['queryRun']['status'] == 'running':
            print("running")
            time.sleep(self.SLEEP_TIMER)

            if attempts < self.MAX_ATTEMPTS:
                results = self._get_query_results(url_endpoint, attempts + 1)
            else:
                # Number of attempts exceeded.  Exit potential infinite loop
                raise Exception('failed after', self.MAX_ATTEMPTS, 'failed attempts')
        # response is an unexpected error
        else:
            raise Exception('server response is', response_data)

        return results

    def _execute_query(self, barrel_token, query_token):
        """
        Executes query. Execute means to run query and get results of query.

        Parameters:
            barrel_token (int): Token the uniquely identifies query barrel. A dictionary with
            all query barrel tokens can be found using get_tab_token_dict(). A tab in the
            DataDistillr user interface is equivalent to a query barrel in API routes and
            responses.

            query_token (int): Token the uniquely identifies query in query barrel.

        Returns:
            pandas dataframe: Formatted results of query.

        """

        # runs query
        query_run_page = self.QUERY_BARRELS + "/" + str(barrel_token) + "/query/" + \
            str(query_token) + "/run"
        query_run = self.session.get(url=query_run_page)
        query_run_json = query_run.json()
        run_request_token = query_run_json["requestToken"]

        # gets result of query
        query_results = self.QUERY_RUN_PAGE + "/" + str(run_request_token)
        attempts = 0
        results = self._get_query_results(query_results, attempts)

        # format query results
        schema = results['summary']['columnNames']
        data = results['data']
        return pd.DataFrame(data, columns=schema)

    def execute_existing_query(self, tab_token):
        """

        Executes most recent query in a tab. The tab is identified by tab_token. A tab in the
        DataDistillr user interface is equivalent to a query barrel in API routes and
        responses.

        Parameters:
            tab_token: Token the uniquely identifies query barrel. A dictionary with
            all tab tokens can be found using get_tab_token_dict().

        Returns:
            pandas dataframe: Formatted results of query.

        """

        query_token = self._get_recent_query_token(tab_token)
        return self._execute_query(tab_token, query_token)

    def execute_new_query(self, tab_name, query):
        """

        Creates new tab named tab_name and executes query in tab.
        A tab in the DataDistillr user interface is equivalent to a query barrel in API routes and
        responses.

        Parameters:
            tab_name (str): Name of new tab
            query (int): SQL statement to be run in tab.

        Returns:
            pandas dataframe: Formatted results of query.
        """

        query_barrel_details = {
            "projectSlug": self.name.lower().replace(' ', '-'),
            "projectToken": self.project_token,
            "name": tab_name,
            "active": True,
            "icon": "type-icon-file",
            "query": "  " + query
        }

        query_barrel_resp = self.session.post(url=self.QUERY_BARRELS, json=query_barrel_details,
                                              verify=False)
        query_barrel_resp_json = query_barrel_resp.json()
        barrel_token = query_barrel_resp_json["queryBarrel"]["queries"][0]["queryBarrelToken"]
        query_token = query_barrel_resp_json["queryBarrel"]["queries"][0]["token"]

        return self._execute_query(barrel_token, query_token)

    def get_data_source_token_dict(self):
        """
        Returns dictionary with data source tokens as keys and data source names as values.

        Returns:
            dictionary (int -> str): Dictionary with data source tokens as keys and data source
            ames as values.
        """

        get_data_sources = self.PROJECT_PAGE + "/" + str(self.project_token) + "/dataSource"
        data_sources_response = self.session.get(url=get_data_sources)
        data_sources_response_json = data_sources_response.json()
        data_sources = data_sources_response_json["dataSources"]
        for data_source in data_sources:
            self.data_source_token_dict[data_source["token"]] = data_source["name"]
        return self.data_source_token_dict

    def get_data_source_token(self, data_source_name):
        """
        Returns data source token that matches data_source_name

        Returns:
            int: data source token
        """

        data_source_token_dict = self.get_data_source_token_dict()
        for token, name in data_source_token_dict.items():
            if data_source_name == name:
                return token
        raise Exception("token not found")

    def _get_presigned_urls(self, data_source_token, file_paths):
        """
        Returns list of AWS presigned urls for each file path in list of file paths

        Returns:
            array (str): list of presigned urls
        """

        # building array of file objects for post request
        files_array = []
        for file_path in file_paths:
            file_size = os.path.getsize(file_path)
            file_name = ntpath.basename(file_path)
            file_obj = {
                "name": file_name,
                "size": file_size,
                "type": "text/csv",
                "path": "/"
            }

            files_array.append(file_obj)
        files = {"files": files_array}

        post_data_source = self.DATA_SOURCE_PAGE + "/" + str(data_source_token) + "/file"
        response = self.session.post(post_data_source, json=files, verify=False)
        presigned_urls = response.json()["presignedUrls"]
        return presigned_urls

    def upload_files(self, data_source_token, file_paths):
        """
        Uploads list of files to a data source.

        Parameters:
            data_source_token (int): Token the uniquely identifies data source.
            file_paths (array): List of absolute file paths of files to be uploaded.

        Returns:
            boolean: True if file was uploaded successfully.
        """

        presigned_urls = self._get_presigned_urls(data_source_token, file_paths)
        for i, file_path in enumerate(file_paths):
            presigned_url = presigned_urls[i]
            with open(file_path, encoding="utf-8") as file:
                file_data = file.read()
                response = self.session.put(presigned_url,
                                            data=file_data,
                                            headers={'content-type': 'text/plain'})

            if not response.ok:
                raise Exception("file not uploaded")
        return 'file uploaded successfully'



"""
This file defines the class for getting data from API Access Clients in Datadistillr account.
"""

import pandas as pd
import requests
from urllib3.exceptions import InsecureRequestWarning
from datadistillr.auth_exceptions import AuthorizationException


class Datadistillr:
    """
    This class is for getting data from API Access Clients in Datadistillr account.
    """

    @staticmethod
    def get_dataframe(url, api_key):
        """
        This function allows you to programmatically access data from DataDistillr and push it to a
        pandas DataFrame. DataDistillr allows you to publish your data by generating an API
        Endpoint. To access your data, you will need an endpoint URL and an Authorization token.
        You can obtain both of these items in DataDistillr under the API Endpoints section.

        If the authorization is not successful this function throws an AuthorizationException.

        Full documentation is available here: https://docs.datadistillr.com/ddr/
        :param url:  Your dataset API URL
        :param api_key: Your unique dataset API key
        :return: A Pandas DataFrame of your data.
        """
        response = Datadistillr.make_api_call(url, api_key)

        schema = response.json()['summary']['columnNames']
        data = response.json()['results']

        # Since we already retrieved the first page, decrement this by 1
        page_count = response.json()['summary']['totalPages'] - 1
        while page_count > 0:
            # Make next API call
            next_url = response.json()['summary']['nextPage']
            response = Datadistillr.make_api_call(next_url, api_key)

            # Append the data
            next_page = response.json()['results']
            data.extend(next_page)
            page_count -= page_count

        return pd.DataFrame(data, columns=schema)

    @staticmethod
    def make_api_call(url, api_key):
        """
        This function allows you to programmatically access data from DataDistillr.
        DataDistillr allows you to publish your data by generating an API Endpoint.
        To access your data, you will need an endpoint URL and an Authorization token. You can
        obtain both of these items in DataDistillr under the API Endpoints section.

        If the authorization is not successful this function throws an AuthorizationException.

        Full documentation is available here: https://docs.datadistillr.com/ddr/
        :param url:  Your dataset API URL
        :param api_key: Your unique dataset API key
        :return: response object from API call.
        """
        headers = {"Authorization": api_key}
        requests.packages.urllib3.disable_warnings(InsecureRequestWarning)
        response = requests.get(url, headers=headers, verify=False)

        # Case for unauthorized access
        if response.status_code in (401, 403):
            raise AuthorizationException(url, "You are not authorized to access this resource.")
        # TODO Add more error response codes

        return response


    @staticmethod
    def get_csv_from_api(url, api_key, filename):
        """
        This function allows you to programmatically access data from DataDistillr and push it to a
        CSV file.DataDistillr allows you to publish your data by generating an API Endpoint.
        To access your data, you will need an endpoint URL and an Authorization token. You can
        obtain both of these items in DataDistillr under the API Endpoints section.

        If the authorization is not successful this function throws an AuthorizationException.

        Full documentation is available here: https://docs.datadistillr.com/ddr/
        :param url:  Your dataset API URL
        :param api_key: Your unique dataset API key
        :param filename: The filename where you want your data written
        :return: A CSV file of your data.
        """
        data_frame = Datadistillr.get_dataframe(url, api_key)
        return data_frame.to_csv(filename)

    @staticmethod
    def get_json_from_api(url, api_key, filename):
        """
        This function allows you to programmatically access data from DataDistillr and push it to a
        JSON file. DataDistillr allows you to publish your data by generating an API Endpoint.
        To access your data, you will need an endpoint URL and an Authorization token. You can
        obtain both of these items in DataDistillr under the API Endpoints section.

        If the authorization is not successful this function throws an AuthorizationException.

        Full documentation is available here: https://docs.datadistillr.com/ddr/
        :param url:  Your dataset API URL
        :param api_key: Your unique dataset API key
        :param filename: The filename where you want your data written
        :return: A JSON file of your data.
        """
        data_frame = Datadistillr.get_dataframe(url, api_key)
        return data_frame.to_json(filename)

    @staticmethod
    def get_parquet_from_api(url, api_key, filename):
        """
        This function allows you to programmatically access data from DataDistillr and push it to a
        parquet file. DataDistillr allows you to publish your data by generating an API Endpoint.
        To access your data, you will need an endpoint URL and an Authorization token. You can
        obtain both of these items in DataDistillr under the API Endpoints section.

        If the authorization is not successful this function throws an AuthorizationException.

        Full documentation is available here: https://docs.datadistillr.com/ddr/
        :param url:  Your dataset API URL
        :param api_key: Your unique dataset API key
        :param filename: The filename where you want your data written
        :return: A parquet file of your data.
        """
        data_frame = Datadistillr.get_dataframe(url, api_key)
        return data_frame.to_parquet(filename)

    @staticmethod
    def get_excel_from_api(url, api_key, filename):
        """
        This function allows you to programmatically access data from DataDistillr and push it to an
        Excel file. DataDistillr allows you to publish your data by generating an API Endpoint.
        To access your data, you will need an endpoint URL and an Authorization token. You can
        obtain both of these items in DataDistillr under the API Endpoints section.

        If the authorization is not successful this function throws an AuthorizationException.

        Full documentation is available here: https://docs.datadistillr.com/ddr/
        :param url:  Your dataset API URL
        :param api_key: Your unique dataset API key
        :param filename: The filename where you want your data written
        :return: An Excel file of your data.
        """
        data_frame = Datadistillr.get_dataframe(url, api_key)
        return data_frame.to_excel(filename)

    @staticmethod
    def get_dict_from_api(url, api_key, filename):
        """
        This function allows you to programmatically access data from DataDistillr and push it to a
        Python dictionary. DataDistillr allows you to publish your data by generating an API
        Endpoint.  To access your data, you will need an endpoint URL and an Authorization token.
        You can obtain both of these items in DataDistillr under the API Endpoints section.

        If the authorization is not successful this function throws an AuthorizationException.

        Full documentation is available here: https://docs.datadistillr.com/ddr/
        :param url:  Your dataset API URL
        :param api_key: Your unique dataset API key
        :param filename: The filename where you want your data written
        :return: A Python dictionary file of your data.
        """
        data_frame = Datadistillr.get_dataframe(url, api_key)
        return data_frame.to_dict(filename)
